// === NOTO SANS ===
@font-face {
    font-family: 'Noto Sans';
    font-style: normal;
    font-weight: normal;
    src: url('/rio/asset/fonts/Noto Sans/NotoSans-Regular.ttf');
}

@font-face {
    font-family: 'Noto Sans';
    font-style: normal;
    font-weight: bold;
    src: url('/rio/asset/fonts/Noto Sans/NotoSans-Bold.ttf');
}

@font-face {
    font-family: 'Noto Sans';
    font-style: italic;
    font-weight: normal;
    src: url('/rio/asset/fonts/Noto Sans/NotoSans-Italic.ttf');
}

@font-face {
    font-family: 'Noto Sans';
    font-style: italic;
    font-weight: bold;
    src: url('/rio/asset/fonts/Noto Sans/NotoSans-BoldItalic.ttf');
}

// === ROBOTO ===
@font-face {
    font-family: 'Roboto';
    font-style: normal;
    font-weight: normal;
    src: url('/rio/asset/fonts/Roboto/Roboto-Regular.ttf');
}

@font-face {
    font-family: 'Roboto';
    font-style: normal;
    font-weight: bold;
    src: url('/rio/asset/fonts/Roboto/Roboto-Bold.ttf');
}

@font-face {
    font-family: 'Roboto';
    font-style: italic;
    font-weight: normal;
    src: url('/rio/asset/fonts/Roboto/Roboto-Italic.ttf');
}

@font-face {
    font-family: 'Roboto';
    font-style: italic;
    font-weight: bold;
    src: url('/rio/asset/fonts/Roboto/Roboto-BoldItalic.ttf');
}

// === ROBOTO MONO ===
@font-face {
    font-family: 'Roboto Mono';
    font-style: normal;
    font-weight: normal;
    src: url('/rio/asset/fonts/Roboto Mono/RobotoMono-Regular.ttf');
}

@font-face {
    font-family: 'Roboto Mono';
    font-style: normal;
    font-weight: bold;
    src: url('/rio/asset/fonts/Roboto Mono/RobotoMono-Bold.ttf');
}

@font-face {
    font-family: 'Roboto Mono';
    font-style: italic;
    font-weight: normal;
    src: url('/rio/asset/fonts/Roboto Mono/RobotoMono-Italic.ttf');
}

@font-face {
    font-family: 'Roboto Mono';
    font-style: italic;
    font-weight: bold;
    src: url('/rio/asset/fonts/Roboto Mono/RobotoMono-BoldItalic.ttf');
}
